package rakete;
/**Die Klasse RK_NumDgl dient zur numerischen Loesung von 
 	Differentialgleichungen erster Ordnung. Es muss eine 
 	Unterklasse abgeleitet werden,in der die abstrakte Funktion

	public double[] fn(double t, double[] x)

	mit der Differentialgleichungsfunktion ueberschrieben wird.

	Konstruktorelemente: int n, double h, double t0, 
							double t1, double[] x0

	n ist die Dimension der Differentialgleichung
	h ist die Schrittweite fuer das RK-Verfahren
	t0 ist der Anfangszeitpunkt (wichtig fuer lsg-Methode und zeitabh.
	   DGls)
	t1 ist der Endzeitpunkt (nur wichtig bei Benutzung der lsg-Methode)
	x0 enthaelt die Anfangsbedingungen

	Die Methode lsg liefert einen zweifach indizierten Loesungsvektor,
	die erste Komponente indiziert die Integrationsschritte, 
	die zweite gibt die Komponenten des jeweiligen Loesungsvektors an.

	Die Methode schritt rechnet mit den momentanen Anfangsbedingungen einen
	RK-Schritt weiter, setzt die Anfangsbedingungen neu und gibt den neuen
	x-Wert zurueck.
	
	@author Fabian Buehler
*/


public abstract class RK_NumDgl extends NumDgl {
   RK_NumDgl(int n, double h, double t0, double t1, double[] x0) {
	 super(n,h,t0,t1,x0);
  }
  private double[] smul(double s, double[] v) {
	 double[] rv= new double[v.length];
	 for (int i=0; i<v.length; i++) {
		rv[i]=s*v[i];
	 }
	 return rv;
  }
  private double[] vadd(double[] v1, double[] v2) {
	 double[] rv= new double[v1.length];
	 for (int i=0; i<v1.length; i++) {
		rv[i]=v1[i]+v2[i];
	 }
	 return rv;
  }
  
  public double[][] lsg() {
	 double[][] lvektor=new double[(int)Math.round((t1-t0)*(1/h)+1)][n];
	 lvektor[0]=x0;
	 for (int i=0; t0+i*h<=t1-h; i++) {
		double[] k1=fn(t0+i*h,lvektor[i]);
		double[] k2=fn(t0+(i+1.0/2.0)*h,vadd(lvektor[i],smul(h/2,k1)));
		double[] k3=fn(t0+(i+1.0/2.0)*h,vadd(lvektor[i],smul(h/2,k2)));
		double[] k4=fn(t0+(i+1)*h,vadd(lvektor[i],smul(h,k3)));
		lvektor[i+1]=vadd(lvektor[i],smul(h,
			  vadd(smul(1.0/6.0,k1),
					 vadd(smul(1.0/3.0,k2),
							vadd(smul(1.0/3.0,k3),smul(1.0/6.0,k4))))));
	 }
	 return lvektor;
  }
  public double[]  schritt() {
	 double[] lvektor=new double[n];
	 double[] k1=fn(t0,x0);
		double[] k2=fn(t0+0.5*h,vadd(x0,smul(h/2,k1)));
		double[] k3=fn(t0+0.5*h,vadd(x0,smul(h/2,k2)));
		double[] k4=fn(t0+1*h,vadd(x0,smul(h,k3)));
		lvektor=vadd(x0,smul(h,
			  vadd(smul(1.0/6.0,k1),
					 vadd(smul(1.0/3.0,k2),
							vadd(smul(1.0/3.0,k3),smul(1.0/6.0,k4))))));
	 x0=lvektor;
	 t0+=h;
	 return lvektor;
  }
}

/*
 * This software is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General 
 * Public License (GPL) as published by the Free Software 
 * Foundation; either version 3 of the License, 
 * or(at your option) any later version.
 *
 * This software is distributed in the hope that it will 
 * be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY 
 * or FITNESS FOR A PARTICULAR PURPOSE.  
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General 
 * Public License along with this; 
 * if not, see <http://www.gnu.org.licenses>
 */
